<?php

namespace Drupal\subscription_manager\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\subscription_manager\SubscriptionManagerService;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Class UserSubscriptionsController.
 *
 * @package Drupal\subscription_manager\Controller
 */
class SubscriptionManagerController extends ControllerBase {

  /**
   * @var \Drupal\subscription_manager\SubscriptionManagerService
   */
  protected $subscriptionManager;

  /**
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  private $logger;

  /**
   * UserSubscriptionsController constructor.
   *
   * @param \Drupal\subscription_manager\SubscriptionManagerService $subscription_manager
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   */
  public function __construct(SubscriptionManagerService $subscription_manager, LoggerChannelInterface $logger, AccountProxyInterface $current_user) {
    $this->subscriptionManager = $subscription_manager;
    $this->logger = $logger;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('subscription_manager'),
      $container->get('logger.channel.subscription_manager'),
      $container->get('current_user'),
    );
  }

  /**
   * Redirect.
   *
   * @return string
   *   Return Hello string.
   */
  public function redirectToSubscriptions() {
    return $this->redirect('subscription_manager.manage_subscription', ['user' => $this->currentUser()->id()]);
  }

  /**
   * @param string $user
   *
   * @return \Drupal\Core\Access\AccessResult
   *   Return
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function manageBillingAccess($user) {
    return AccessResult::allowedIf($this->subscriptionManager->userHasLocalSubscriptionToAnyPlan(User::load($user)));
  }

  /**
   * @param $user
   *
   * @return TrustedRedirectResponse
   */
  public function manageBilling($user): TrustedRedirectResponse {
    $subscription = $this->subscriptionManager->getUserSubscriptionToAnyPlan(User::load($user));
    $connector_plugin_id = $subscription->getConnectorPluginId();
    /** @var \Drupal\subscription_manager\SubscriptionManagerConnectorInterface $connector */
    $connector = $this->subscriptionManager->getConnectorManager()->createInstance($connector_plugin_id);
    return $connector->redirectToPortal($user);
  }

  /**
   * @return string
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function subscribeTitle(): string {
    if ($this->subscriptionManager->userHasLocalSubscriptionToAnyPlan($this->currentUser())) {
      return 'Upgrade';
    }
    return 'Subscribe';
  }

  /**
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function subscribe() {
    $user = User::load($this->currentUser()->id());
    // If user already has a subscription, send them to the corresponding connector's subscribe page.
    if ($subscription = $this->subscriptionManager->getUserSubscriptionToAnyPlan($user)) {
      $connector_plugin_id = $subscription->getConnectorPluginId();
    }
    else {
      $connector_plugin_id = $this->config('subscription_manager.settings')->get('default_connector');
    }

    /** @var \Drupal\subscription_manager\SubscriptionManagerConnectorInterface $connector */
    $connector = $this->subscriptionManager->getConnectorManager()->createInstance($connector_plugin_id);
    return $connector->redirectToSubscribe($user);
  }

  /**
   * @param \Symfony\Component\HttpFoundation\Request $request
   *
   * @return \Symfony\Component\HttpFoundation\Response
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \JsonException
   */
  public function getMySubscription(Request $request): Response {
    $subscription = $this->subscriptionManager->getUserSubscriptionToAnyPlan(User::load($this->currentUser()->id()));
    if ($subscription) {
      $data = [
        'subscription' => $subscription->toArray()
      ];
    }
    else {
      $data = [
        'subscription' => FALSE,
      ];
    }

    return new Response(json_encode($data, JSON_THROW_ON_ERROR), Response::HTTP_ACCEPTED);
  }

}
